package edu.rice.comp322;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

/**
 * This class should not be modified.
 *
 * @author <a href="http://shams.web.rice.edu/">Shams Imam</a> (shams@rice.edu)
 */
public class SudokuConfig {

    public static final String NODES_EXPLORED = "NodesExplored";
    private static final String DEFAULT_FILE_NAME = "src/main/resources/sudoku/16x16-1.txt";
    private static final ThreadLocalCounter counter = new ThreadLocalCounter();

    protected static int granularity = 1_000;
    protected static String fileName = DEFAULT_FILE_NAME;

    protected static ProblemState initialProblemState = null;

    protected static void parseArgs(final String[] args) throws IOException {
        int i = 0;

        while (i < args.length) {
            final String loopOptionKey = args[i];

            switch (loopOptionKey) {
                case "-f":
                    i += 1;
                    fileName = args[i];
                    break;
                case "-g":
                    i += 1;
                    granularity = Integer.parseInt(args[i]);
                    break;
            }

            i += 1;
        }

        boardInit();
    }

    protected static void printArgs() {
        System.out.printf(BenchmarkRunner.argOutputFormat, "Input File Name", SudokuConfig.fileName);
        System.out.printf(BenchmarkRunner.argOutputFormat, "Granularity", granularity);
    }

    protected static void boardInit() throws IOException {

        final File inputFile = new File(fileName);

        if (!inputFile.exists()) {
            System.out.printf("Couldn't open %s file for reading\n", fileName);
            System.exit(1);
        }

        /* read input file and initialize */
        final int[] validValues = read(fileName);
        initialProblemState = new ProblemState(validValues);
    }

    /*
     * Reads a single sudoku puzzle from a file
     * using the format from
     * http://sudoku.cvs.sourceforge.net/viewvc/sudoku/puzzles/
     * but generalized to support larger board sizes
     */
    public static int[] read(final String fileName) {
        final File file = new File(fileName);
        final StringBuilder contents = new StringBuilder();
        BufferedReader reader = null;
        int lineSize = -1;
        try {
            reader = new BufferedReader(new FileReader(file));
            String text = null;

            // repeat until all lines is read
            while ((text = reader.readLine()) != null) {
                //System.out.println("Reading.");
                if (text.length() > 0) {
                    if (text.charAt(0) != '/') {
                        lineSize = text.length();
                        contents.append(text);
                        contents.append(System.getProperty(
                                "line.separator"));
                    }
                }
            }
        } catch (final IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (final IOException e) {
                e.printStackTrace();
            }
        }

        // show file contents here
        // System.out.println(contents.toString());

        int noValues = 0;
        for (int i = 0; i < lineSize; i++) {
            if ((contents.charAt(i) == '|')
                    || (contents.charAt(i) == '+')
                    || (contents.charAt(i) == '-')
                    || (contents.charAt(i) == ' ')
                    || (contents.charAt(i) == '\n')
                    || (contents.charAt(i) == '\r')) {
            } else {
                noValues++;
            }
        }
        final int[] board = new int[noValues * noValues];

        int crtPos = 0;
        for (int i = 0; i < contents.length(); i++) {

            if ((contents.charAt(i) == '|')
                    || (contents.charAt(i) == '+')
                    || (contents.charAt(i) == '-')
                    || (contents.charAt(i) == ' ')
                    || (contents.charAt(i) == '\n')
                    || (contents.charAt(i) == '\r')) {
                // do nothing
            } else {

                switch (contents.charAt(i)) {
                    case '.':
                        board[crtPos++] = -1;
                        break;
                    case '0':
                        board[crtPos++] = 0;
                        break;
                    case '1':
                        board[crtPos++] = 1;
                        break;
                    case '2':
                        board[crtPos++] = 2;
                        break;
                    case '3':
                        board[crtPos++] = 3;
                        break;
                    case '4':
                        board[crtPos++] = 4;
                        break;
                    case '5':
                        board[crtPos++] = 5;
                        break;
                    case '6':
                        board[crtPos++] = 6;
                        break;
                    case '7':
                        board[crtPos++] = 7;
                        break;
                    case '8':
                        board[crtPos++] = 8;
                        break;
                    case '9':
                        board[crtPos++] = 9;
                        break;
                    default:
                        final char c = contents.charAt(i);
                        board[crtPos++] = (c < 'a') ? c - 'A' + 10 : c - 'a' + 10;
                        break;
                }

            }
        }
        return board;
    }

    protected static void incrementCount() {
        counter.increment();
    }

    protected static void validate(final Benchmark benchmark, final ProblemState problemState) {

        if (problemState == null) {
            System.out.println("ERROR: No solution found!");
        } else if (!problemState.isSolution()) {
            System.out.println("ERROR: Computed board is not a valid solution!");
        }

        final long nodesExplored = counter.get();
        benchmark.track(NODES_EXPLORED, nodesExplored);
        System.out.printf(BenchmarkRunner.argOutputFormat, "Nodes explored", nodesExplored);
        counter.reset();
    }

}
