package edu.rice.comp322;

import java.util.*;

/**
 * @author <a href="http://shams.web.rice.edu/">Shams Imam</a> (shams@rice.edu)
 */
public final class SudokuUtils {

    private SudokuUtils() {
        super();
    }

    protected static List<Integer> fillPossibleValues(
            final int location,
            final int boardSize,
            final List<Integer>[] board) {

        // System.out.printf("Location %2d: ", location);
        final boolean[] takenValues = new boolean[boardSize];

        final int itemRowIndex = location / boardSize;
        final int itemColIndex = location % boardSize;
        final int miniBoardSize = (int) Math.sqrt(boardSize);
        {
            // first mark values from rows
            final int rowStartIndex = itemRowIndex;
            for (int i = rowStartIndex * boardSize, j = 0; j < boardSize; i++, j++) {
                final List<Integer> loopSetValues = board[i];
                // System.out.printf(" %2d", i);

                if (loopSetValues.size() == 1) {
                    final Integer loopValue = loopSetValues.get(0);
                    takenValues[loopValue] = true;
                }
            }
        }
        // System.out.printf(" : ");
        {
            // next mark values from cols
            final int colIndex = itemColIndex;
            for (int i = 0, j = colIndex; i < boardSize; i++, j += boardSize) {
                final List<Integer> loopSetValues = board[j];
                // System.out.printf(" %2d", j);

                if (loopSetValues.size() == 1) {
                    takenValues[loopSetValues.get(0)] = true;
                }
            }
        }
        // System.out.printf(" : ");
        {
            // finally mark values from mini-boxes
            final int rowStartIndex = itemRowIndex - (itemRowIndex % miniBoardSize);
            final int colStartIndex = (itemColIndex / miniBoardSize) * miniBoardSize;
            for (int i = rowStartIndex, i1 = 0; i1 < miniBoardSize; i++, i1++) {
                for (int j = colStartIndex, j1 = 0; j1 < miniBoardSize; j++, j1++) {
                    final int boardIndex = (i * boardSize) + j;
                    final List<Integer> loopSetValues = board[boardIndex];
                    // System.out.printf(" %2d", boardIndex);

                    if (loopSetValues.size() == 1) {
                        takenValues[loopSetValues.get(0)] = true;
                    }
                }
            }
        }
        // System.out.printf("\n");

        final List<Integer> resultList = new ArrayList<>();
        for (int i = 0; i < takenValues.length; i++) {
            if (!takenValues[i]) {
                resultList.add(i);
            }
        }
        return resultList;
    }

    /**
     * @param location         The location where the {@code newValue} needs to be placed
     * @param newValue         A newValue to set into the board at the specified {@code location}
     * @param boardSize        The size of the board, same as {@code board.length}
     * @param board            The board (free value table)
     * @param mutatedLocations A list to store the mutated locations as a result of setting the {@code newValue} at
     *                         {@code location}
     */
    protected static void setPossibleValue(
            final int location,
            final int newValue,
            final int boardSize,
            final List<Integer>[] board,
            final List<Integer> mutatedLocations) {

        // Ensure that board[location] == newValue. DO NOT make recursive calls to setPossibleValue().
        // Hint: Look at fillPossibleValues() to determine how to work on the row, column, and mini-boxes.

        // TODO: set the board location to the new value by clearing out previous entries


        final int itemRowIndex = location / boardSize;
        final int itemColIndex = location % boardSize;
        final int miniBoardSize = (int) Math.sqrt(boardSize);
        {
            // first mark values from rows
            // TODO Ensure none of the other positions for the row have newValue as an entry
        }
        {
            // next mark values from cols
            // TODO Ensure none of the other positions for the column have newValue as an entry
        }
        {
            // finally mark values from mini-boxes
            // TODO Ensure none of the other positions for the mini-box have newValue as an entry
        }
    }

    protected static int countUnsolved(final List<Integer>[] board) {

        // TODO return the number of unsolved entries. A position is unsolved if it has greater than 1 entry.
        int count = 0;
        return count;
    }

    protected static boolean isSolution(final int numValues, final List<Integer>[] board) {

        // TODO Implement this. A board is solved if all the lists have a size of 1.

        return true;
    }

    /**
     * @param numValues        The size of the board, same as {@code board.length}
     * @param board            The board (free value table)
     * @param mutatedLocations The list of mutate locations from calls to setPossibleValue(). Since the previous board
     *                         state was expected to be valid, only changes to these locations can cause the board to
     *                         become invalid. Hence, these are the locations that need to be checked in part (b). The
     *                         mutatedLocations can tell you which rows, columns, and miniboxes need to be checked.
     * @return true if the board is consistent, false otherwise.
     */
    protected static boolean isConsistent(final int numValues, final List<Integer>[] board, final List<Integer> mutatedLocations) {

        // TODO Implement this.
        // mutatedLocations provides an optimization opportunity by limiting the locations to check.
        // A board is consistent if:
        // (a) none of the lists have a size lesser than 1,
        // (b) there are no duplicates in a given row, column, and mini-box.

        return true;
    }

    protected static <T> Queue<T> newWorkQueue(final Comparator<T> comparator) {
        // TODO return an appropriate work queue
        return new LinkedList<>();
    }

    protected static Comparator<ProblemState> boardComparator() {
        // TODO This comparator is incorrect; the assignment requires lexicographic comparisons of boards
        return (o1, o2) -> {
            if (o1 == null) {
                return -1;
            }
            return o2.numMoves - o1.numMoves;
        };
    }

    protected static <T> Queue<T>[] splitTaskQueue(final Queue<T> queue, final Comparator<T> comparator, final int numSplits) {

        @SuppressWarnings("unchecked")
        final Queue<T>[] splitQueues = new Queue[numSplits];
        for (int i = 0; i < splitQueues.length; i++) {
            splitQueues[i] = SudokuUtils.newWorkQueue(comparator);
        }

        // TODO split the queue into the two queues evenly
        splitQueues[0].addAll(queue);
        queue.clear();

        return splitQueues;
    }

    protected static int granularityIncrement(final int granularity, final int maxGranularity) {
        return Math.min(2 * granularity, maxGranularity);
    }
}
