package edu.rice.comp322;

import edu.rice.hj.api.HjRunnable;
import junit.framework.TestCase;

import java.util.Comparator;

import static edu.rice.hj.Module0.launchHabaneroApp;

/**
 * This is a test class for your homework and should not be modified.
 *
 * @author <a href="http://shams.web.rice.edu/">Shams Imam</a> (shams@rice.edu)
 * @author Vivek Sarkar (vsarkar@rice.edu)
 */
public class Homework4ParallelizationTest extends TestCase {

    public void testEurekaSearch9x9v1() {
        System.out.println("Homework4ParallelizationTest.testEurekaSearch9x9v1: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/9x9-1.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelSearchBenchmark();

        kernelFunction(inputArgs, parBench, false);

        System.out.println("Homework4ParallelizationTest.testEurekaSearch9x9v1: ends.");
    }

    public void testEurekaSearch9x9v3() {
        System.out.println("Homework4ParallelizationTest.testEurekaSearch9x9v3: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/9x9-3.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelSearchBenchmark();

        kernelFunction(inputArgs, parBench, false);

        System.out.println("Homework4ParallelizationTest.testEurekaSearch9x9v3: ends.");
    }

    public void testEurekaSearch16x16v2() {
        System.out.println("Homework4ParallelizationTest.testEurekaSearch16x16v2: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/16x16-2.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelSearchBenchmark();

        kernelFunction(inputArgs, parBench, false);

        System.out.println("Homework4ParallelizationTest.testEurekaSearch16x16v2: ends.");
    }

    public void testEurekaSearch16x16v3() {
        System.out.println("Homework4ParallelizationTest.testEurekaSearch16x16v3: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/16x16-3.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelSearchBenchmark();

        kernelFunction(inputArgs, parBench, false);

        System.out.println("Homework4ParallelizationTest.testEurekaSearch16x16v3: ends.");
    }


    public void testEurekaLexical9x9v1() {
        System.out.println("Homework4ParallelizationTest.testEurekaLexical9x9v1: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/9x9-1.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelLexicalBenchmark();

        kernelFunction(inputArgs, parBench, true);

        System.out.println("Homework4ParallelizationTest.testEurekaLexical9x9v1: ends.");
    }

    public void testEurekaLexical9x9v3() {
        System.out.println("Homework4ParallelizationTest.testEurekaLexical9x9v3: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/9x9-3.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelLexicalBenchmark();

        kernelFunction(inputArgs, parBench, true);

        System.out.println("Homework4ParallelizationTest.testEurekaLexical9x9v3: ends.");
    }

    public void testEurekaLexical16x16v2() {
        System.out.println("Homework4ParallelizationTest.testEurekaLexical16x16v2: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/16x16-2.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelLexicalBenchmark();

        kernelFunction(inputArgs, parBench, true);

        System.out.println("Homework4ParallelizationTest.testEurekaLexical16x16v2: ends.");
    }

    public void testEurekaLexical16x16v3() {
        System.out.println("Homework4ParallelizationTest.testEurekaLexical16x16v3: starts...");

        final String[] inputArgs = {"-f", "src/main/resources/sudoku/16x16-3.txt", "-g", "1000"};
        final Benchmark parBench = new ParallelLexicalBenchmark();

        kernelFunction(inputArgs, parBench, true);

        System.out.println("Homework4ParallelizationTest.testEurekaLexical16x16v3: ends.");
    }


    private void kernelFunction(final String[] inputArgs, final Benchmark parBench, final boolean validateMoves) {

        try {
            SudokuConfig.parseArgs(inputArgs);
        } catch (final Exception ex) {
            ex.printStackTrace();
            throw new RuntimeException(ex);
        }

        final ProblemState[] parResult = {null};
        timeIt("Parallel", () -> {
            launchHabaneroApp(() -> {
                System.out.println("  Running: " + parBench.name());
                parResult[0] = parBench.computeSudoku();
            });
        });

        final ProblemState[] seqResult = {null};
        timeIt("Sequential", () -> {
            launchHabaneroApp(() -> {
                final Benchmark seqBench = new SequentialBenchmark();
                System.out.println("  Running: " + seqBench.name());
                seqResult[0] = seqBench.computeSudoku();
            });
        });

        assertTrue("ERROR: Parallel solution returned null", parResult[0] != null);
        System.out.println("  Parallel solution is not null.");

        if (validateMoves) {
            final Comparator<ProblemState> comparator = SudokuUtils.boardComparator();
            final int compareResult = comparator.compare(parResult[0], seqResult[0]);
            assertTrue("ERROR: Parallel solution not better than Sequential solution.", compareResult >= 0);
            System.out.println("  Parallel solution better than Sequential solution.");
        }

        assertTrue("ERROR: Parallel solutions is not consistent", parResult[0].isConsistent());
        System.out.println("  Parallel solution is consistent.");

        assertTrue("ERROR: Parallel solutions is not a solution", parResult[0].isSolution());
        System.out.println("  Parallel solution is a solution.");
    }

    private static void timeIt(final String name, final HjRunnable runnable) {
        final long startTime = System.nanoTime();
        runnable.run();
        final long endTime = System.nanoTime();
        final long execTimeMillis = (long) ((endTime - startTime) / 1e6);
        System.out.println(" " + name + " took " + execTimeMillis + "ms.");
    }
}
